# -*- coding: utf-8 -*-
#MoxJitter 161024 増減機能修正。timeStep,Valueに誤差範囲機能追加,　ギザギザモード削除
import maya.cmds as cmds
import random


def main():
    
    oSels = cmds.ls( sl=True )#選択したオブジェクトを取得
    for oSel in oSels:#選択したノードを個別に処理、つまりoSelはセレクト
        oFcvs = cmds.keyframe(oSel,q=True, n=True, sl=True) #選択したオブジェクトの選択したFカーブを取得
        if not oFcvs:#Fカーブリストに何も入らなかった場合
                cmds.error(u"FCurveが選択されていません。")
                
        for oFcv in oFcvs:#取得したFカーブリストから、一本だけＦカーブを取り出し個別に処理
            #print(u"Fカーブの名前   "+ oFcv)
            sampleByTime = cmds.intField("step", q=True, v=True)
            oSelMinKey = cmds.keyframe(q=True,tc=True,sl=True)[0] #選択範囲の最初のキー
    	    oSelMaxKey = cmds.keyframe(q=True,tc=True,lsl=True)[-1] #選択範囲の最後のキー
            #print(u"最初のキー " +str(oSelMinKey) + u"最後のキー "+str(oSelMaxKey))
            
            cmds.bufferCurve (oFcv, animation ="keys", overwrite= False)
            cmds.bakeResults(oFcv, sampleBy=sampleByTime, t=(oSelMinKey, oSelMaxKey), 
                        preserveOutsideKeys=True)#指定間隔でキーを打つ
            
            listKeyVal = cmds.keyframe(oFcv, q=True, vc=True)#vc= キーの値をリスト化。全部
            listKeyTim = cmds.keyframe(oFcv, q=True, tc=True)#tc= キーの時間をリスト化。もちろん全部
            iNumKey = cmds.keyframe(oFcv, q=True, kc=True)#kc= キーフレームの数
            #print(listKeyTim)
            
            #各フィールドから値を取得します。
            stepT = cmds.intField("stepTolerance", q=True, v=True) #ステップの許容範囲を取得
            val = cmds.floatField("value", q=True, v=True)#Valueを取得
            valSplit = val / (iNumKey-2)#Valueを配置フレームの数で割る
            ValTole = cmds.floatField("valueTolerance", q=True, v=True)#値の許容範囲値を取得  
            ValToleSplit = ValTole / (iNumKey-2)#V値の許容範囲値を配置フレームの数で割る
            
            for i in range(iNumKey):#キーフレームの数だけ繰り返す。
                if oSelMinKey < listKeyTim[i] and listKeyTim[i] < oSelMaxKey: #選択した指定範囲内であれば
                    
                    randT = random.randint( abs(stepT)*-1, abs(stepT) )#時間許容値からランダムな整数値を取得
                    ValToleRand = random.uniform(abs(ValTole)*-1, abs(ValTole))#値の許容範囲値からランダムな値を取得
                    ValToleRandSplit =  ValToleRand / (iNumKey-2)#配置フレームの数で割る
                    
                    #増減の判定
                    if cmds.radioCollection("fluctuation", q=True, sl=True) == "Flat":pass
                    elif cmds.radioCollection("fluctuation",  q=True, sl=True) == "Increase":
                        val = valSplit * i
                        ValToleRand = ValToleRandSplit * i
                    elif cmds.radioCollection("fluctuation",  q=True, sl=True) == "Decrease":
                        if i == 1:pass
                        else: 
                            val = val - valSplit
                            ValToleRand = ValToleRand - ValToleRandSplit
                   
                    #print(str(listKeyTim[i]) + "   " + str(randT) + "=" + str(listKeyTim[i] +randT))

                    if i % 2 == 1:#奇数なら減算
                        cmds.cutKey(oFcv, t=(listKeyTim[i],), clear=True)
                        cmds.setKeyframe(oFcv, t=listKeyTim[i]+randT, v=(listKeyVal[i]+ abs(val)*-1 + ValToleRand) )
   
                    else:#偶数なら加算
                        cmds.cutKey(oFcv, t=(listKeyTim[i],), clear=True)
                        cmds.setKeyframe(oFcv, t=listKeyTim[i]+randT, v=(listKeyVal[i]+ abs(val)+ ValToleRand) )
            
                
#UI
class MJmain(object):
    def __init__(self):
        if cmds.window("MoxJitter",ex=True):cmds.deleteUI("MoxJitter") 
        cmds.window("MoxJitter",tlb=True)
        cmds.columnLayout(adjustableColumn=True, columnAttach=["both",5] )
        
        cmds.rowLayout(nc=4)
        cmds.text("step", l="timestep")
        cmds.intField("step",value=2, w=30, step=1, min=1, ann=u"キーを何フレーム間隔で打つかを指定します。")
        cmds.text("stepTolerance", l="+-")
        cmds.intField("stepTolerance", value=0, w=30, step=1, min=0, ann=u"フレーム間隔に誤差最大値を指定します。")
        cmds.setParent("..")
        
        cmds.rowLayout(nc=4)
        cmds.text("value")
        cmds.floatField("value", v=1.0, ann=u"値の範囲を指定します。", step=1, min=0.1, pre=1, w=30)
        
        cmds.text("valueTolerance", l="+-")    
        cmds.floatField("valueTolerance", v=0, ann=u"値の範囲における誤差最大値を指定します。", en=True,  step=1, min=0, pre=1, w=30)
        cmds.setParent("..")
       
        cmds.radioCollection("fluctuation")
        cmds.radioButton( 'Flat', ann=u"増減の変化を指定しません")
        cmds.radioButton( 'Increase', ann=u"徐々に増加していき、終点でValueで指定した値に達します。" )
        cmds.radioButton( 'Decrease', ann=u"Valueで指定した値から始まり、徐々に減少していきます。" )
        cmds.radioCollection("fluctuation", edit=True, select="Flat" )    
        
        cmds.button("Run", c="MoxJitter.main()" )
        cmds.showWindow("MoxJitter")
mainWind = MJmain()
      